#ifndef INFERENCE_QUADRATIC_SOLVER_PARAMETERS_H__
#define INFERENCE_QUADRATIC_SOLVER_PARAMETERS_H__

#include "LinearSolverParameters.h"

class QuadraticSolverParameters : public LinearSolverParameters {};

#endif // INFERENCE_QUADRATIC_SOLVER_PARAMETERS_H__

