#ifndef INFERENCE_RELATION_H__
#define INFERENCE_RELATION_H__

/** Used to indicate the relation of a linear constraint.
 */
enum Relation {

	LessEqual,
	Equal,
	GreaterEqual
};

#endif // INFERENCE_RELATION_H__

